"use strict";
/**
 * @module "@blitzm/kubernetes-nginx-ingress"
 * @packageDocumentation
 * @example
 * ```typescript
 * import { KubernetesNginxIngress } from '@blitzm/kubernetes-nginx-ingress';
 *
 * new KubernetesNginxIngress('my-thing', {
 *   // ...
 * });
 * ```
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const pulumi = __importStar(require("@pulumi/pulumi"));
const k8s = __importStar(require("@pulumi/kubernetes"));
const path = __importStar(require("path"));
const lodash_1 = require("lodash");
/**
 * @noInheritDoc
 */
class KubernetesNginxIngress extends pulumi.ComponentResource {
    constructor(name, props, opts) {
        var _a, _b;
        super('cloud:kubernetes:KubernetesNginxIngress', name, props, opts);
        this.ingressClass = pulumi.output('nginx');
        const ingressChart = new k8s.helm.v3.Chart(name, {
            namespace: props === null || props === void 0 ? void 0 : props.namespace,
            path: path.join(__dirname, 'charts/ingress-nginx'),
            values: {
                fullnameOverride: name,
                rbac: {
                    create: true,
                },
                defaultBackend: {
                    enabled: false,
                },
                controller: lodash_1.merge({}, this.controllerValues(props), {
                    config: props === null || props === void 0 ? void 0 : props.config,
                    labels: {
                        app: 'nginx-ingress',
                    },
                    podLabels: {
                        app: 'nginx-ingress',
                    },
                    admissionWebhooks: {
                        enabled: false,
                    },
                    metrics: {
                        enabled: true,
                        service: {
                            type: 'ClusterIP',
                            servicePort: 10254,
                            annotations: {
                                'prometheus.io/scrape': 'true',
                                'prometheus.io/port': '10254',
                            },
                        },
                    },
                    extraArgs: this.extraArgs(props),
                    maxmindLicenseKey: props === null || props === void 0 ? void 0 : props.maxmindLicenseKey,
                }),
                tcp: this.l4Services((_a = props === null || props === void 0 ? void 0 : props.tcpServices) !== null && _a !== void 0 ? _a : {}),
                udp: this.l4Services((_b = props === null || props === void 0 ? void 0 : props.udpServices) !== null && _b !== void 0 ? _b : {}),
            },
        }, {
            parent: this,
            provider: props === null || props === void 0 ? void 0 : props.provider,
        });
        const serviceName = `${name}-controller`;
        this.ingressService = (props === null || props === void 0 ? void 0 : props.namespace) ? ingressChart.getResource('v1/Service', props.namespace.toString(), serviceName)
            : ingressChart.getResource('v1/Service', serviceName);
    }
    controllerValues(props) {
        var _a;
        switch ((_a = props === null || props === void 0 ? void 0 : props.mode) === null || _a === void 0 ? void 0 : _a.kind) {
            case 'DaemonSet':
                return this.daemonSetValues(props === null || props === void 0 ? void 0 : props.mode);
            case 'Deployment':
                return this.deploymentValues(props === null || props === void 0 ? void 0 : props.mode);
            default:
                return this.deploymentValues({
                    kind: 'Deployment',
                    replicas: 1,
                });
        }
    }
    daemonSetValues(props) {
        return {
            kind: 'DaemonSet',
            reportNodeInternalIp: true,
            service: {
                enabled: false,
            },
            publishService: {
                enabled: false,
            },
            hostNetwork: true,
            hostPort: {
                enabled: true,
            },
        };
    }
    deploymentValues(props) {
        var _a;
        const replicas = (_a = props === null || props === void 0 ? void 0 : props.replicas) !== null && _a !== void 0 ? _a : 1;
        return {
            kind: 'Deployment',
            replicaCount: replicas,
            service: {
                type: 'LoadBalancer',
                loadBalancerIP: props === null || props === void 0 ? void 0 : props.loadBalancerIP,
                externalTrafficPolicy: 'Local',
                annotations: props.serviceAnnotations,
            },
            /**
             * we'll only add the antiaffinity policy if
             * there are more than 1 replicas
             */
            affinity: replicas <= 1
                ? undefined
                : {
                    podAntiAffinity: {
                        preferredDuringSchedulingIgnoredDuringExecution: [
                            {
                                weight: 1,
                                podAffinityTerm: {
                                    topologyKey: 'kubernetes.io/hostname',
                                    labelSelector: {
                                        matchExpressions: [
                                            {
                                                key: 'app',
                                                operator: 'In',
                                                values: ['nginx-ingress'],
                                            },
                                            {
                                                key: 'component',
                                                operator: 'In',
                                                values: ['controller'],
                                            },
                                        ],
                                    },
                                },
                            },
                        ],
                    },
                },
        };
    }
    l4Services(props) {
        const backends = {};
        for (const [key, value] of Object.entries(props)) {
            backends[Number(key)] = pulumi.interpolate `${value.namespace}/${value.serviceName}:${value.servicePort}`;
        }
        return backends;
    }
    extraArgs(props) {
        const args = {};
        if (props === null || props === void 0 ? void 0 : props.enableSSLPassthrough) {
            // the --enable-ssl-passthrough flag doesn't take a value
            args['enable-ssl-passthrough'] = '';
        }
        return args;
    }
}
exports.KubernetesNginxIngress = KubernetesNginxIngress;
//# sourceMappingURL=index.js.map